@@ -145,7 +145,6 @@ int CmdShow::execute (std::string& output)
     " debug"
     " debug.hooks"
     " debug.parser"
-    " debug.tls"
     " default.command"
     " default.due"
     " default.project"
@@ -165,6 +164,7 @@ int CmdShow::execute (std::string& output)
     " fontunderline"
     " gc"
     " hooks"
+    " hooks.location"
     " hyphenate"
     " indent.annotation"
     " indent.report"
@@ -192,14 +192,13 @@ int CmdShow::execute (std::string& output)
     " search.case.sensitive"
     " sugar"
     " summary.all.projects"
+    " sync.local.server_dir"
+    " sync.gcp.credential_path"
+    " sync.gcp.bucket"
+    " sync.server.client_id"
+    " sync.encryption_secret"
+    " sync.server.origin"
     " tag.indicator"
-    " taskd.server"
-    " taskd.ca"
-    " taskd.certificate"
-    " taskd.ciphers"
-    " taskd.credentials"
-    " taskd.key"
-    " taskd.trust"
     " undo.style"
     " urgency.active.coefficient"
     " urgency.scheduled.coefficient"
