@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -62,25 +62,11 @@ int CmdStats::execute (std::string& output)
 
   std::string dateformat = Context::getContext ().config.get ("dateformat");
 
-  // Go get the file sizes.
-  size_t dataSize = Context::getContext ().tdb2.pending._file.size ()
-                  + Context::getContext ().tdb2.completed._file.size ()
-                  + Context::getContext ().tdb2.undo._file.size ()
-                  + Context::getContext ().tdb2.backlog._file.size ();
-
-  // Count the undo transactions.
-  std::vector <std::string> undoTxns = Context::getContext ().tdb2.undo.get_lines ();
-  int undoCount = 0;
-  for (auto& tx : undoTxns)
-    if (tx == "---")
-      ++undoCount;
+  // Count the possible reverts.
+  int undoCount = Context::getContext ().tdb2.num_reverts_possible ();
 
   // Count the backlog transactions.
-  std::vector <std::string> backlogTxns = Context::getContext ().tdb2.backlog.get_lines ();
-  int backlogCount = 0;
-  for (auto& tx : backlogTxns)
-    if (tx[0] == '{')
-      ++backlogCount;
+  int numLocalChanges = Context::getContext ().tdb2.num_local_changes ();
 
   // Get all the tasks.
   Filter filter;
@@ -123,13 +109,13 @@ int CmdStats::execute (std::string& output)
     if (task.is_blocked)  ++blockedT;
     if (task.is_blocking) ++blockingT;
 
-    time_t entry = strtol (task.get ("entry").c_str (), nullptr, 10);
+    time_t entry = strtoll (task.get ("entry").c_str (), nullptr, 10);
     if (entry < earliest) earliest = entry;
     if (entry > latest)   latest   = entry;
 
     if (status == Task::completed)
     {
-      time_t end = strtol (task.get ("end").c_str (), nullptr, 10);
+      time_t end = strtoll (task.get ("end").c_str (), nullptr, 10);
       daysPending += (end - entry) / 86400.0;
     }
 
@@ -203,17 +189,13 @@ int CmdStats::execute (std::string& output)
   view.set (row, 0, "Blocking tasks");
   view.set (row, 1, blockingT);
 
-  row = view.addRow ();
-  view.set (row, 0, "Data size");
-  view.set (row, 1, formatBytes (dataSize));
-
   row = view.addRow ();
   view.set (row, 0, "Undo transactions");
   view.set (row, 1, undoCount);
 
   row = view.addRow ();
   view.set (row, 0, "Sync backlog transactions");
-  view.set (row, 1, backlogCount);
+  view.set (row, 1, numLocalChanges);
 
   if (totalT)
   {
